# -*- coding: utf-8 -*-
import os
from django.core.urlresolvers import reverse, NoReverseMatch
from django.db import models
from django.utils.translation import ugettext_lazy as _
from mediastore.conf import settings
from mediastore.models import Media


def get_file_extension(file):
    bits = file.name.split('.',-1)
    if len(bits) == 1:
        return ''
    else:
        return bits[-1].lower()

class Download(Media):
    help_text = _('Stores an downloadable file, its file extension and the file size.')

    file = models.FileField(_('download file'),
        upload_to=os.path.join(settings.MEDIASTORE_FS_PREFIX, 'downloads')
    )
    file_extension = models.CharField(_('file extenstion'), max_length=12,
        null=True, blank=True, editable=False)
    file_size = models.PositiveIntegerField(_('file size'), editable=False)
    count = models.PositiveIntegerField(_(u'download counter'), default=0)

    class Meta:
        app_label = 'mediastore'
        verbose_name = _('download')
        verbose_name_plural = _('downloads')

    def save(self, *args, **kwargs):
        self.file_extension = get_file_extension(self.file)
        self.file_size = self.file.size
        super(Download, self).save(*args, **kwargs)

    def get_download_link(self):
        try:
            url = reverse('mediastore-download-link', args=(
                self.slug, os.path.basename(self.file.name)))
        except NoReverseMatch, e:
            url = self.file.url
        return url



from django import template

def _order_by(queryset, value):
    args = value.split(',')
    return queryset.order_by(*args)

def _offset(queryset, value):
    value = int(value)
    return queryset[value:]

def _limit(queryset, value):
    value = int(value)
    return queryset[:value]

QUERYSET_OPTIONS_CALLBACK = {
    'order_by': _order_by,
    'offset': _offset,
    'limit': _limit,
}


class ValidationError(Exception):
    pass


def parse_queryset_option_value(value):
    '''
    Options that get parsed:

    datetime(now)
    datetime(2009-02-23 03:10:59)
    '''
    if value.startswith('datetime(') and value.endswith(')'):
        from datetime import datetime
        value = value[len('datetime('):-1]
        if value == 'now':
            value = datetime.now()
        else:
            value = datetime.strptime(value, '%Y-%m-%d %H:%M:%s')
    return value


def parse_queryset_options(string):
    '''
    >>> parse_queryset_options('order_by="-created,-pub_date",title__exact="Hello world!",pub_date__lte=datetime(now)')
    [['order_by', '-created'], ['title__exact', 'Hello world!'], ['pub_date__lte', datetime.now()]
    '''
    options = []
    reset, finish = True, False
    open_token = None
    key, value = '', ''
    i = -1
    while True:
        i += 1
        try:
            s = string[i]
        except IndexError:
            reset = True
            finish = True
        if reset:
            if key:
                value = parse_queryset_option_value(value)
                options.append([key, value])
            mode = 'key'
            key, value = '', ''
            reset = False
            if finish:
                break
        if mode == 'key':
            if s == '=':
                if not key:
                    raise ValidationError
                mode = 'value'
                continue
            if s == ',':
                reset = True
                mode = 'key'
            else:
                key += s
        if mode == 'value':
            if not open_token and s == ',':
                reset = True
                continue
            if not open_token and s in ["'", '"']:
                open_token = s
                continue
#            if s == open_token and not masked(open_token, i, s):
            if s == open_token:
                open_token = None
                reset = True
                continue
            value += s
    return options


def apply_queryset_options(queryset, options, callbacks=None):
    if callbacks is None:
        callbacks = {}
    for key, value in options:
        if key in callbacks:
            queryset = callbacks[key](queryset, value)
        elif key in QUERYSET_OPTIONS_CALLBACK:
            queryset = QUERYSET_OPTIONS_CALLBACK[key](queryset, value)
        else:
            if value.startswith('-'):
                queryset = queryset.exclude(**{key: value[:-1]})
            else:
                queryset = queryset.filter(**{key: value})
    return queryset


class GetModelsNode(template.Node):
    def __init__(self, as_value, queryset):
        self.as_value = as_value
        self.queryset = queryset

    def render(self, context):
        context[self.as_value] = self.queryset
        return ''


def get_models(queryset, queryset_options=None):
    QUERYSET = queryset
    def func(parser, token):
        '''
        Example:
            {% get_models as object_list with order_by="-created" %}
        '''
        try:
            tokens = token.contents.split(None, 4)
        except ValueError:
            raise template.TemplateSyntaxError(u'%r tag requires two or four arguments.' % token.contents.split()[0])
        if len(tokens) == 3:
            tokens += [None, None]
        tag_name, as_, as_value, with_, options = tokens
        if as_ != 'as':
            raise template.TemplateSyntaxError(u'%r\'s second argument must be "as".' % tag_name)
        queryset = QUERYSET
        if options is not None:
            options = parse_queryset_options(options)
            queryset = apply_queryset_options(queryset, options, queryset_options)
        if with_ is not None and with_ != 'with':
            raise template.TemplateSyntaxError(u'%r\'s fourth argument must be "with".' % tag_name)
        return GetModelsNode(as_value, queryset)
    return func



# -*- coding: utf-8 -*-
from django.db import models
from sortedm2m.fields import SortedManyToManyField
from mediastore.forms import MediaMultipleChoiceField
from mediastore.widgets import MediaSelect, MediaSelectMultiple


class MediaField(models.ForeignKey):
    def __init__(self, **kwargs):
        from mediastore.models import Media
        kwargs['to'] = Media
        super(MediaField, self).__init__(**kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'widget': MediaSelect(
                rel=self.rel,
                **kwargs.pop('widget_kwargs', {})),
        }
        defaults.update(kwargs)
        return super(MediaField, self).formfield(**defaults)


class MultipleMediaField(SortedManyToManyField):
    def __init__(self, to=None, sorted=False, *args, **kwargs):
        if to is None:
            from mediastore.models import Media
            to = Media
        super(MultipleMediaField, self).__init__(to, sorted, *args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': MediaMultipleChoiceField,
            'rel': self.rel,
            'widget': MediaSelectMultiple(
                rel=self.rel,
                sorted=self.sorted,
                **kwargs.pop('widget_kwargs', {})),
            'sorted': self.sorted,
        }
        defaults.update(kwargs)
        return super(MultipleMediaField, self).formfield(**defaults)


try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    add_introspection_rules = None

if add_introspection_rules:
    add_introspection_rules([
        (
            [MediaField],
            [],
            {},
        ),
    ], ["^mediastore\.fields\.related\.MediaField"])


    add_introspection_rules([
        (
            [MultipleMediaField],
            [],
            {'to': ('rel.to', {'default': 'mediastore.Media'})},
        ),
    ], ["^mediastore\.fields\.related\.MultipleMediaField"])



# -*- coding: utf-8 -*-
import os
from django.utils.translation import ugettext_lazy as _
from django.db import models
from mediastore.conf import settings
from mediastore.models import Media


THUMBNAIL_UPLOAD_DIR = os.path.join(
    settings.MEDIASTORE_FS_PREFIX, 'embeded/thumbnails')


class Embeded(Media):
    content = models.TextField()
    thumbnail = models.ImageField(
        upload_to=THUMBNAIL_UPLOAD_DIR,
        null=True, blank=True)

    class Meta:
        app_label = 'mediastore'
        verbose_name = _('embeded media')
        verbose_name_plural = _('embeded media')



# -*- coding: utf-8 -*-

def order_queryset_by_pks(queryset, pks):
    object_list = dict((
        (unicode(key), value)
        for key, value in queryset.in_bulk(pks).iteritems()))
    return [object_list[unicode(pk)] for pk in pks]



# -*- coding: utf-8 -*-
import os
from django.utils.translation import ugettext_lazy as _
from django.db import models
from mediastore.conf import settings
from mediastore.models import Media


class SWF(Media):
    help_text = _('Stores a swf file.')

    file = models.FileField(_('swf file'),
        upload_to=os.path.join(settings.MEDIASTORE_FS_PREFIX, 'swf')
    )
    width = models.PositiveIntegerField(_('width'), null=True, blank=True)
    height = models.PositiveIntegerField(_('height'), null=True, blank=True)

    class Meta:
        app_label = 'mediastore'
        verbose_name = _('SWF')
        verbose_name_plural = _('SWFs')



# -*- coding: utf-8 -*-
class SettingsProxy(object):
    def __init__(self, settings, defaults):
        self.settings = settings
        self.defaults = defaults

    def __getattr__(self, attr):
        try:
            return getattr(self.settings, attr)
        except AttributeError:
            try:
                return getattr(self.defaults, attr)
            except AttributeError:
                raise AttributeError, 'settings object has no attribute "%s"' % attr


class defaults(object):
    MEDIASTORE_FS_PREFIX = 'media'


from django.conf import settings as django_settings
settings = SettingsProxy(django_settings, defaults)



import sys, os
from django.conf import settings
from mediastore.mediatypes.video.models import QueueItem


LOCK_FILE = getattr(settings, 'MEDIASTORE_VIDEO_LOCK_FILE', '/tmp/mediastore_video_convert.lock')


def get_epoch():
    import datetime, time
    return time.mktime(datetime.datetime.now().timetuple())


def get_next_item():
    previous = None
    while True:
        current = QueueItem.objects.get_next()
        if current is None:
            break
        if current == previous:
            print "detected endless loop"
            sys.exit(1)
        previous = current
        yield current


def lock(update=False):
    if update == False and os.path.exists(LOCK_FILE):
        print "It seems that there is already a process running " +\
            "which proccesses the video queue. Please check and " +\
            "try again. If you are sure there is no process " +\
            "running, you can delete the " + LOCK_FILE + " file " +\
            "or call the command with the argument 'unlock' again."
        return False
    f = file(LOCK_FILE, 'w')
    f.write(str(get_epoch()))
    f.close()
    return True


def unlock():
    if os.path.exists(LOCK_FILE):
        os.unlink(LOCK_FILE)


def process_queue():
    # if we cannot get the lock, abort
    if not lock():
        return
    try:
        for item in get_next_item():
            item.process()
            lock(update=True)
    finally:
        unlock()


def process_queue_threaded():
    import threading
    thread = threading.Thread(target=process_queue)
    thread.daemon = True
    thread.start()
    return thread


def process_queue_systemprocess():
    import multiprocessing
    process = multiprocessing.Process(target=process_queue)
    process.daemon = True
    process.start()
    return process




from django import forms
from django.contrib import admin
from mediastore.admin import MediaAdmin
from mediastore.mediatypes.pdf.models import PDF


class PDFAdmin(MediaAdmin):
    list_display = ('id', 'name', 'file_size', 'created')


admin.site.register(PDF, PDFAdmin)



# -*- coding: utf-8 -*-
from django import forms
from sortedm2m.forms import SortedMultipleChoiceField
from mediastore.models import Media
from mediastore.widgets import MediaSelect, MediaSelectMultiple


__all__ = (
    'MediaChoiceField',
    'MediaMultipleChoiceField')


class MediaChoiceField(forms.ModelChoiceField):
    def __init__(self, *args, **kwargs):
        mediatype = kwargs.pop('mediatype', None)
        queryset = Media.objects.all()
        if mediatype:
            queryset = queryset.type(mediatype)
        kwargs.setdefault('queryset', queryset)
        from django.db import models
        if mediatype:
            db_field = models.ForeignKey(Media, limit_choices_to={'type': mediatype})
        else:
            db_field = models.ForeignKey(Media)
        kwargs.setdefault('widget', MediaSelect(db_field.rel, mediatype=mediatype))
        super(MediaChoiceField, self).__init__(*args, **kwargs)


class MediaMultipleChoiceField(SortedMultipleChoiceField):
    def __init__(self, *args, **kwargs):
        self.rel = kwargs.pop('rel')
        self.sorted = kwargs.pop('sorted', False)
        mediatype = kwargs.pop('mediatype', None)
        queryset = Media.objects.all()
        if mediatype:
            queryset = queryset.type(mediatype)
        kwargs.setdefault('queryset', queryset)
        kwargs.setdefault('widget', MediaSelectMultiple(rel=self.rel, mediatype=mediatype, sorted=self.sorted))
        super(MediaMultipleChoiceField, self).__init__(*args, **kwargs)



# -*- coding: utf-8 -*-
import os
import operator
import tempfile
from mediastore.mediatypes.video.convert.base import VideoConverterBase
from mediastore.mediatypes.video.conf import settings


class FFmpegConverter(VideoConverterBase):
    convert_bin = settings.MEDIASTORE_VIDEO_FFMPEG_BIN
    convert_args = settings.MEDIASTORE_VIDEO_FFMPEG_ARGS
    thumbnail_bin = settings.MEDIASTORE_VIDEO_FFMPEG_THUMBNAIL_BIN
    thumbnail_args = settings.MEDIASTORE_VIDEO_FFMPEG_THUMBNAIL_ARGS

    class ConvertionError(VideoConverterBase.ConvertionError):
        pass

    def prepare_config(self, config):
        default_config = {
            'croptop': '0',
            'cropbottom': '0',
            'cropleft': '0',
            'cropright': '0',
            'padtop': '0',
            'padbottom': '0',
            'padleft': '0',
            'padright': '0',
        }
        config = super(FFmpegConverter, self).prepare_config(config)
        default_config.update(config)
        return default_config

    def convert(self, input_path, output_path, extra_config=None):
        extra_config = {}
        expected_width = int(self.config['width'])
        expected_height = int(self.config['height'])

        # create sample thumbnail to get the size of the video
        tfile = tempfile.mkstemp()[1]
        thumbnail_args = [
            '-i', input_path,
            '-f', 'mjpeg',
            '-ss', '0',
            '-an',
            '-y', tfile,
        ]
        try:
            try:
                from PIL import Image
            except ImportError:
                import Image
        except ImportError, e:
            self.stderr += (
                'cannot import PIL. You need PIL installed '
                'to use proportional scaling with ffmpeg backend.\n')
            raise self.ConvertionError
        self.execute_command(self.prepare_command_args(
            self.thumbnail_bin,
            thumbnail_args,
        ))
        try:
            actual_width, actual_height = Image.open(tfile).size
        except IOError, e:
            self.stderr += e.args[0] + '\n'
            raise self.ConvertionError
        # delete sample thumbnail
        os.unlink(tfile)

        extra_args = []

        # now keep the width and crop or pad height to get the correct
        # proportions
        expected_ratio = operator.truediv(expected_width, expected_height)
        actual_ratio = operator.truediv(actual_width, actual_height)

        if settings.MEDIASTORE_VIDEO_FFMPEG_CROP:
            # video is wider than expected video dimensions
            if expected_ratio < actual_ratio:
                wanted_width = int(round(
                    actual_height * expected_ratio, 0))
                crop = actual_width - wanted_width
                crop -= crop % 4
                extra_config['cropleft'] = crop / 2
                extra_config['cropright'] = crop / 2
            # video is higher than expected video dimensions
            else:
                wanted_height = int(round(
                    actual_width * expected_ratio ** -1, 0))
                crop = actual_height - wanted_height
                crop -= crop % 4
                extra_config['croptop'] = crop / 2
                extra_config['cropbottom'] = crop / 2

        # don't crop so we pad
        else:
            # video is wider than expected video dimensions
            if expected_ratio < actual_ratio:
                wanted_height = int(round(
                    expected_width * actual_ratio ** -1, 0))
                padding = expected_height - wanted_height
                padding -= padding % 4
                self.config['padtop'] = padding / 2
                self.config['padbottom'] = padding / 2
                wanted_height -= wanted_height % 2
                self.config['height'] = str(wanted_height)
            # video is higher than expected video dimensions
            elif expected_ratio > actual_ratio:
                wanted_width = int(round(
                    expected_height * actual_ratio, 0))
                padding = expected_width - wanted_width
                padding -= padding % 4
                self.config['padleft'] = padding / 2
                self.config['padright'] = padding / 2
                wanted_width -= wanted_width % 2
                self.config['width'] = str(wanted_width)

        super(FFmpegConverter, self).convert(
            input_path, output_path, extra_config)



# -*- coding: utf-8 -*-
from django.db.models import F
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from mediastore.mediatypes.download.models import Download


def download_counter(request, slug, filename=None):
    download = get_object_or_404(Download, slug=slug)
    Download.objects.filter(pk=download.pk).update(count=F('count') + 1)
    return HttpResponseRedirect(download.file.url)



# -*- coding: utf-8 -*-
from django import forms
from django.contrib import admin
from django.contrib.admin import SimpleListFilter
from django.contrib.admin.util import unquote
from django.core import urlresolvers
from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.utils.translation import ugettext_lazy as _
from mediastore import media_url
from mediastore.fields import MediaField, MultipleMediaField
from mediastore.models import Media
from mediastore.preview import display_media
from mediastore.utils import get_media_models, get_model_for_type
from mediastore.utils.http import HttpJsonResponse
from mediastore.utils.models import order_queryset_by_pks
from mediastore.widgets import MediaTypeSelect


def admin_preview_media(*args, **kwargs):
    kwargs['template_name_single'] = 'admin/mediastore/media/object_preview.html'
    kwargs['template_name_many'] = 'admin/mediastore/media/many_object_preview.html'
    return display_media(*args, **kwargs)


class MediaTypeFilter(SimpleListFilter):
    title = _(u'Mediatype')
    parameter_name = 'content_type__model'

    def lookups(self, request, model_admin):
        types = []
        for model in get_media_models():
            types.append((
                model._meta.object_name.lower(),
                model._meta.verbose_name))
        return types

    def queryset(self, request, queryset):
        if 'content_type__model' in self.used_parameters:
            lookup = self.used_parameters['content_type__model']
            queryset = queryset.filter(content_type__model=lookup)
        return queryset



class SelectedFilter(SimpleListFilter):
    title = _(u'Selected')
    parameter_name = 'pk__in'

    def lookups(self, request, model_admin):
        return (
            ('', _('Selected')),
        )

    def queryset(self, request, queryset):
        if 'pk__in' in self.used_parameters:
            lookup = self.used_parameters['pk__in']
            lookup = [l for l in lookup.split(',') if l]
            queryset = queryset.filter(pk__in=lookup)
        return queryset


class MediaAdmin(admin.ModelAdmin):
    list_display = ('id', 'preview', 'name', 'created')
    list_display_links = ('id', 'name')
    search_fields = ('name', 'description')
    date_hierarchy = 'created'
    prepopulated_fields = {'slug': ('name',)}
    save_on_top = True
    actions_on_bottom = True

    def preview(self, obj):
        from mediastore.preview import preview_media
        return preview_media(obj, {'size':'100px'})
    preview.allow_tags = True

    MAIN_FIELDSET = (None, {
        'fields': ('name', 'description', 'slug', 'tags')
    })


class ModelAdmin(admin.ModelAdmin):
    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        if isinstance(db_field, MediaField):
            # prevent the + (add object) symbol from appearing
            self.raw_id_fields += (db_field.name,)
            return db_field.formfield(
                widget_kwargs={'admin_site': self.admin_site},
                **kwargs)
        return super(ModelAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    def formfield_for_manytomany(self, db_field, request=None, **kwargs):
        if isinstance(db_field, MultipleMediaField):
            # prevent the + (add object) symbol from appearing
            self.raw_id_fields += (db_field.name,)
            return db_field.formfield(
                widget_kwargs={'admin_site': self.admin_site},
                **kwargs)
        return super(ModelAdmin, self).formfield_for_manytomany(db_field, request, **kwargs)


class MediaModelAdmin(admin.ModelAdmin):
    select_template = None

    list_display = ('slug', 'preview', 'name', 'content_type', 'created')
    list_display_links = ('slug', 'name')
    search_fields = ('name', 'description')
    date_hierarchy = 'created'
    list_filter = (MediaTypeFilter, SelectedFilter,)

    select_list_per_page = 25

    def preview(self, obj):
        from mediastore.preview import preview_media
        return preview_media(obj.object, {'size':'100px'})
    preview.allow_tags = True

    class add_form(forms.ModelForm):
        type = forms.CharField(
            label=_('Select the media type you want to add'),
            widget=MediaTypeSelect()
        )

        class Meta:
            model = Media
            fields = ('type',)
        class Media:
            js = (
                media_url('js/jquery-1.3.2.min.js'),
                media_url('js/mediatype_select.js'),
            )
            css = {
                'all': (media_url('css/mediatype_select.css'),)
            }

        def save(self, *args, **kwargs):
            raise NotImplementedError()
        def clean_type(self):
            self.cleaned_data['app_label'], \
            self.cleaned_data['model_name'] = \
                self.cleaned_data['type'].split('.', 1)
            return self.cleaned_data

    def add_view(self, request, *args, **kwargs):
        if request.GET.has_key('type'):
            model = get_model_for_type(request.GET['type'])
            url = urlresolvers.reverse(
                '%s:%s_%s_add' % (
                    self.admin_site.name,
                    model._meta.app_label.lower(),
                    model._meta.object_name.lower(),
                )
            )
            if request.REQUEST.has_key('_popup'):
                url += '?_popup=1'
            return HttpResponseRedirect(url)
        change_form = self.form
        try:
            self.form = self.add_form
            kwargs.setdefault('extra_context', {}).\
                setdefault('media_models', get_media_models())
            media_models = [
                model for model in get_media_models()
                if model in self.admin_site._registry
            ]
            field = self.form.base_fields['type']
            field_widget = field.widget
            field.widget = MediaTypeSelect(media_models=media_models)
            if request.method == 'POST':
                form = self.form(request.POST)
                if form.is_valid():
                    # TODO: continue work
                    admin_name = self.admin_site.name
                    # TODO: make url dynamic
                    url = urlresolvers.reverse(
                        '%s:%s_%s_add' % (
                            self.admin_site.name,
                            form.cleaned_data['app_label'],
                            form.cleaned_data['model_name'],
                        )
                    )
                    if request.REQUEST.get('_popup', None):
                        url += '?_popup=1'
                    return HttpResponseRedirect(url)
            response = super(MediaModelAdmin, self).\
                add_view(request, *args, **kwargs)
            field.widget = field_widget
            return response
        finally:
            self.form = change_form

    def change_view(self, request, object_id, extra_context=None):
        '''
        Redirects to the changeview of the correct model.
        '''
        # TODO(gregor@muellegger.de): find a way to automatically link back to
        # the media changelist after editing the specific model.
        try:
            media = self.queryset(request).get(pk=unquote(object_id))
            obj = media.object
            return HttpResponseRedirect(urlresolvers.reverse(
                '%s:%s_%s_change' % (
                    self.admin_site.name,
                    obj._meta.app_label,
                    obj._meta.object_name.lower(),
                ),
                args=(object_id,),
            ))
        except Media.DoesNotExist:
            response = super(MediaModelAdmin, self).\
                change_view(request, object_id, extra_context)
            return response

    def changelist_view(self, request, extra_context=None):
        if request.GET.get('pop', 0):
            return self.select_view(request, extra_context)
        return super(MediaModelAdmin, self).changelist_view(request, extra_context)

    def select_view(self, request, extra_context=None):
        if '_get_previews' in request.GET:
            try:
                pks = [int(pk) for pk in request.GET['_get_previews'].split(',')]
            except ValueError:
                pks = None
            if pks is not None:
                previews = []
                queryset = self.model._default_manager.filter(pk__in=pks)
                queryset = order_queryset_by_pks(queryset, pks)
                for obj in queryset:
                    previews.append({
                        'id': obj.pk,
                        'preview': admin_preview_media(obj),
                    })
                return HttpJsonResponse(previews)

        response = super(MediaModelAdmin, self).changelist_view(request, extra_context)
        if isinstance(response, TemplateResponse):
            opts = self.model._meta
            response.template_name = [
                'admin/%s/%s/select.html' % (opts.app_label, opts.object_name.lower()),
                'admin/%s/select.html' % opts.app_label,
                'admin/select.html'
            ]
        return response

    def select_many_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context.update({
            'multiple': True,
        })
        return self.select_view(request, extra_context)

    def get_urls(self):
        from django.conf.urls import patterns, url
        urlpatterns = super(MediaModelAdmin, self).get_urls()
        return patterns('',
            url(r'^select/$',
                self.admin_site.admin_view(self.select_view),
                name='mediastore_media_select'),
            url(r'^select-many/$',
                self.admin_site.admin_view(self.select_many_view),
                name='mediastore_media_select_many'),
        ) + urlpatterns

    def lookup_allowed(self, lookup, value):
        if lookup in ('content_type__model', 'content_type__model__in'):
            return True
        return super(MediaModelAdmin, self).lookup_allowed(lookup, value)


admin.site.register(Media, MediaModelAdmin)



# -*- coding: utf-8 -*-
from django.contrib import admin
from mediastore.admin import MediaAdmin
from mediastore.mediatypes.set.models import Set


class SetAdmin(MediaAdmin):
    list_display = (
        'id',
        'preview',
        'name',
        'created')

admin.site.register(Set, SetAdmin)



from django.contrib.contenttypes.models import ContentType
from django.db.models import get_apps, get_models, get_app, get_model


def unique_slug(text, model, slug_field='slug'):
    from django.template.defaultfilters import slugify
    base = slug = slugify(text)
    suffix = 0
    while True:
        if suffix:
            slug = "-".join([base, str(suffix)])
        if not model.objects.filter(**{'%s__exact' % slug_field: slug}).count():
            return slug
        suffix += 1


def is_media_model(model):
    from mediastore.models import Media
    if model is Media:
        return False
    return issubclass(model, Media)


def is_media_instance(model):
    from mediastore.models import Media
    if model.__class__ is Media:
        return False
    return isinstance(model, Media)


def get_media_models(app_label=None, model_label=None):
    if app_label:
        apps = [get_app(app_label)]
    else:
        apps = get_apps()
    models = []
    for app in apps:
        if model_label:
            app_label = app.__name__.split('.', -1)[-2]
            models.append(get_model(app_label, model_label))
        else:
            models.extend(get_models(app))
    models = [model for model in models if is_media_model(model)]
    return models


def get_media_types(app_label=None, model_label=None):
    models = get_media_models(app_label, model_label)
    types = set()
    for model in models:
        if model.media_type:
            types.add(model.media_type)
    return types


def get_model_for_type(media_type):
    for model in get_media_models():
        if model.media_type == media_type:
            return model
    raise ValueError('There is no such mediatype available.')


def get_content_types_for_type(media_type):
    models = []
    for model in get_media_models():
        if model.media_type == media_type:
            models.append(
                ContentType.objects.get_for_model(model))
    return models


def get_preview(obj, size):
    from django.template import TemplateDoesNotExist
    from django.template.loader import render_to_string
    data = {
        'object': obj,
        'size': size,
    }
    try:
        return render_to_string(
            'mediastore/types/%s/preview.html' % obj.object.media_type,
            data
        )
    except TemplateDoesNotExist:
        return render_to_string('mediastore/no_display_found.html', data)



def get_convert_backend():
    from mediastore.mediatypes.video.conf import settings
    bits = settings.MEDIASTORE_VIDEO_BACKEND.split('.')
    module, obj = '.'.join(bits[:-1]), bits[-1]
    module = __import__(module, globals(), locals(), [obj], -1)
    backend = getattr(module, obj)
    return backend



# -*- coding: utf-8 -*-
import os
import stat
import shutil
import zipfile
from datetime import datetime
from django.utils.translation import ugettext_lazy as _
from django import forms
from django.core.exceptions import PermissionDenied
from django.db import transaction
from django.contrib import admin
from django.contrib.admin import helpers
from django.contrib.contenttypes.models import ContentType
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django import template
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from functools import update_wrapper
from taggit.forms import TagField
from mediastore.admin import MediaAdmin
from mediastore.conf import settings
from mediastore.mediatypes.image.models import Image

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO


class BatchUploadForm(forms.Form):
    archive = forms.FileField(label=_('Zip archive with images'))
    name_scheme = forms.CharField(
        label=_('Naming scheme'),
        help_text=_(
            'Naming scheme for batch renaming. It must contain '
            '(but only once) the "#" sign which is replaced by '
            'a continous number. If it is empty, the original filename '
            'will be taken as name.'
        ),
        max_length=250,
        required=False
    )
    tags = TagField(
        label=_('Tags'), required=False,
        help_text=_(
            'Specify a set of tags that are assigned to all new generated '
            'images.')
    )

    def clean_archive(self):
        zip1 = StringIO()
        for line in self.cleaned_data['archive'].chunks():
            zip1.write(line)
        try:
            zip2 = zipfile.ZipFile(zip1)
        except zipfile.BadZipfile:
            raise forms.ValidationError(_('Please supply a valid zipfile.'))
        if zip2.testzip() is not None:
            raise forms.ValidationError(_('Please supply a valid zipfile.'))
        return zip2

    def clean_name_scheme(self):
        if self.data['name_scheme'] == '':
            return ''
        elif self.data['name_scheme'].count('#') != 1:
            raise forms.ValidationError, _(
                'Name scheme must either contain one "#" sign or must be '
                'empty.')
        else:
            return self.data['name_scheme']

    def get_namelist(self):
        archive = self.cleaned_data['archive']
        return archive.namelist()

    def get_file(self, filename):
        archive = self.cleaned_data['archive']
        return archive.read(filename)

    def apply_name_scheme(self, filename, count):
        name_scheme = self.cleaned_data['name_scheme']
        if not name_scheme:
            return filename
        return name_scheme.replace('#', force_unicode(count))

    def extract_to_file(self, filename, destdir):
        try:
            from PIL import Image
        except ImportError:
            import Image

        archive = self.cleaned_data['archive']
        data = archive.read(filename)
        file_data = StringIO(data)
        try:
            Image.open(file_data)
        except:
            return False
        outfile = os.path.join(destdir, filename.replace('/', '__'))
        # check if file would overwrite existing one
        while os.path.exists(outfile):
            name, ext = os.path.splitext(outfile)
            outfile = '%s_%s' % (name, ext)
        dirname = os.path.dirname(outfile)
        if not os.path.exists(dirname):
            os.makedirs(dirname, 0775)
        photofile = file(outfile, 'wb')
        photofile.write(file_data.getvalue())
        photofile.close()
        if outfile.startswith(settings.MEDIA_ROOT):
            outfile = outfile[len(settings.MEDIA_ROOT):]
            if outfile.startswith('/'):
                outfile = outfile[1:]
        return outfile

    def save(self, model_class):
        destdir = os.path.join(
            settings.MEDIA_ROOT,
            settings.MEDIASTORE_FS_PREFIX,
            'images',
        )
        object_list = []
        count = 1
        for filename in self.get_namelist():
            name = self.apply_name_scheme(filename, count)
            imagefile = self.extract_to_file(filename, destdir)
            if imagefile:
                obj = model_class(
                    name=name,
                    file=imagefile,
                )
                obj.save()
                obj.tags.set(*self.cleaned_data['tags'])
                count += 1
                object_list.append(obj)
        return object_list


class ImageAdmin(MediaAdmin):
    list_display = (
        'id',
        'preview',
        'name',
        'width',
        'height',
        'mimetype',
        'created')
    list_filter = ('mimetype',)

    add_many_form = BatchUploadForm
    add_many_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': (
                'archive',
                'name_scheme',
                'tags',
            ),
        }),
    )

    def render_add_many_form(self, request, context, form_url=''):
        opts = self.model._meta
        app_label = opts.app_label
        add = add_many = True
        change = False
        context.update({
            'add': add,
            'add_many': add_many,
            'change': change,
            'has_add_permission': self.has_add_permission(request),
            'has_change_permission': self.has_change_permission(request),
            'has_delete_permission': self.has_delete_permission(request),
            'has_file_field': True, # FIXME - this should check if form or formsets have a FileField,
            'has_absolute_url': hasattr(self.model, 'get_absolute_url'),
            'form_url': mark_safe(form_url),
            'opts': opts,
            'content_type_id': ContentType.objects.get_for_model(self.model).id,
            'save_as': False,
            'save_on_top': False,
        })
        context_instance = template.RequestContext(request,
            current_app=self.admin_site.name)
        return render_to_response(self.change_form_template or [
            "admin/%s/%s/add_many_form.html" % (app_label, opts.object_name.lower()),
            "admin/%s/add_many_form.html" % app_label,
            "admin/add_many_form.html"
        ], context, context_instance=context_instance)

    def add_many_view(self, request, form_url='', extra_context=None):
        "The 'add' admin view for this model."
        model = self.model
        opts = model._meta

        if not self.has_add_permission(request):
            raise PermissionDenied

        ModelForm = self.add_many_form
        formsets = []
        if request.method == 'POST':
            form = ModelForm(request.POST, request.FILES)
            if form.is_valid():
                object_list = form.save(model)
                self.message_user(request, _(
                    '%(count)d %(verbose_name_plural)s were added successfully.') % {
                        'count': len(object_list),
                        'verbose_name_plural': force_unicode(
                            opts.verbose_name_plural)})
                if self.has_change_permission(request, None):
                    post_url = '../'
                else:
                    post_url = '../../../'
                return HttpResponseRedirect(post_url)
        else:
            form = ModelForm()

        adminForm = helpers.AdminForm(form, self.add_many_fieldsets, {})
        media = self.media + adminForm.media

        inline_admin_formsets = []

        context = {
            'title': _('Add multiple %s') % force_unicode(opts.verbose_name_plural),
            'adminform': adminForm,
            'is_popup': request.REQUEST.has_key('_popup'),
            'show_delete': False,
            'media': mark_safe(media),
            'inline_admin_formsets': inline_admin_formsets,
            'errors': helpers.AdminErrorList(form, formsets),
            'app_label': opts.app_label,
        }
        if extra_context:
            context.update(extra_context)
        return self.render_add_many_form(request, context, form_url=form_url)
    add_many_view = transaction.atomic(add_many_view)

    def get_urls(self):
        from django.conf.urls import patterns, url
        urlpatterns = super(ImageAdmin, self).get_urls()

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        urlpatterns = patterns('',
            url(r'^addmany/$',
                wrap(self.add_many_view),
                name='%s_%s_addmany' % info),
        ) + urlpatterns
        return urlpatterns


admin.site.register(Image, ImageAdmin)



# -*- coding: utf-8 -*-
from django.contrib import admin
from mediastore.admin import MediaAdmin
from mediastore.mediatypes.embeded.models import Embeded


class EmbededAdmin(admin.ModelAdmin):
    pass


admin.site.register(Embeded, EmbededAdmin)




# -*- coding: utf-8 -*-
import os
from django.contrib.admin.templatetags.admin_static import static
from django.utils.translation import ugettext as _
from django.conf import settings
from django.core.urlresolvers import reverse
from django import forms
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from mediastore import media_url
from mediastore.preview import preview_media
from mediastore.models import Media
from mediastore.utils import get_media_models
from mediastore.utils.models import order_queryset_by_pks

__all__ = ('MediaSelect', 'MediaSelectMultiple')


MEDIASELECT_CLASS = 'vForeignKeyMediaAdminField'
MEDIASELECTMULTIPLE_CLASS = 'vManyToManyMediaAdminField'


def _set_class(attrs, class_name):
    class_ = attrs.get('class', '')
    if class_:
        class_ += ' '
    class_ += class_name
    attrs['class'] = class_
    return attrs

def _get_string_as_list(value):
    if type(value) in (unicode, str):
        return (value,)
    return value


def _get_previews(values, field_name):
    output = []
    output.append('<div class="previews">')
    objs = order_queryset_by_pks(
        Media.objects.filter(pk__in=values),
        values)
    for obj in objs:
        output.append('<div class="preview preview-%s-%d">' % (field_name, obj.pk))
        output.append(preview_media(obj, {'size':'160px'}))
        output.append('<a href="javascript:deleteMediaRelation(\'%s\', \'id_%s\', \'%d\');" class="deletelink-media">' % (field_name, field_name, obj.pk))
        output.append('</a>')
        output.append('</div>')
    output.append('</div>')
    return output


class _AdminSiteDummy(object):
    _registry = {}


from django.contrib.admin.widgets import ForeignKeyRawIdWidget
class MediaSelect(ForeignKeyRawIdWidget):
    class Media:
        css = {
            'all': (media_url('css/select.css'),),
        }
        js = (
            media_url('js/jquery-1.3.2.min.js'),
            media_url('js/mediaselect.js'),
        )

    def __init__(self, rel, admin_site=_AdminSiteDummy(), *args, **kwargs):
        self.mediatype = kwargs.pop('mediatype', None)
        super(MediaSelect, self).__init__(rel, admin_site, *args, **kwargs)

    def url_parameters(self):
        params = super(MediaSelect, self).url_parameters()
        if self.mediatype:
            params['mediatype'] = self.mediatype
        return params

    def render(self, name, value, attrs=None):
        rel_to = self.rel.to
        if attrs is None:
            attrs = {}
        extra = []
        if rel_to in self.admin_site._registry:
            # The related object is registered with the same AdminSite
            related_url = reverse('admin:%s_%s_select' %
                                    (rel_to._meta.app_label,
                                    rel_to._meta.module_name),
                                    current_app=self.admin_site.name)

            params = self.url_parameters()
            if params:
                url = u'?' + u'&amp;'.join([u'%s=%s' % (k, v) for k, v in params.items()])
            else:
                url = u''
            if "class" not in attrs:
                attrs['class'] = 'vForeignKeyRawIdAdminField' # The JavaScript code looks for this hook.
            # TODO: "lookup_id_" is hard-coded here. This should instead use
            # the correct API to determine the ID dynamically.
            extra.append(u'<a href="%s%s" class="related-lookup" id="lookup_id_%s" onclick="return showMediaObjectLookupPopup(this);"> '
                            % (related_url, url, name))
            extra.append(u'<img src="%s" width="16" height="16" alt="%s" /></a>'
                            % (static('admin/img/selector-search.gif'), _('Lookup')))
        output = [super(ForeignKeyRawIdWidget, self).render(name, value, attrs)] + extra
        if value:
            output.append(self.label_for_value(value))
        output.extend(_get_previews([value] if value else [], name))
        return mark_safe(u''.join(output))


from django.contrib.admin.widgets import ManyToManyRawIdWidget
class MediaSelectMultiple(ManyToManyRawIdWidget):
    class Media:
        css = {
            'all': (media_url('css/select.css'),),
        }
        js = (
            media_url('js/jquery-1.3.2.min.js'),
            media_url('js/jquery-ui-1.7.2.custom.min.js'),
            media_url('js/mediaselect.js'),
        )

    def __init__(self, rel, admin_site=_AdminSiteDummy(), *args, **kwargs):
        self.mediatype = kwargs.pop('mediatype', None)
        self.sorted = kwargs.pop('sorted', None)
        super(MediaSelectMultiple, self).__init__(rel, admin_site, *args, **kwargs)

    def url_parameters(self):
        params = self.base_url_parameters()
        if self.mediatype:
            params['mediatype'] = self.mediatype
        return params

    def render(self, name, value, attrs):
        rel_to = self.rel.to
        if attrs is None:
            attrs = {}
        extra = []
        if rel_to in self.admin_site._registry:
            # The related object is registered with the same AdminSite
            related_url = reverse('admin:%s_%s_select_many' %
                                    (rel_to._meta.app_label,
                                    rel_to._meta.module_name),
                                    current_app=self.admin_site.name)

            params = self.url_parameters()
            if params:
                url = u'?' + u'&amp;'.join([u'%s=%s' % (k, v) for k, v in params.items()])
            else:
                url = u''
            if "class" not in attrs:
                attrs['class'] = 'vManyToManyRawIdAdminField' # The JavaScript code looks for this hook.
            # TODO: "lookup_id_" is hard-coded here. This should instead use
            # the correct API to determine the ID dynamically.
            extra.append(u'<a href="%s%s" class="related-lookup" id="lookup_id_%s" onclick="return showMediaObjectLookupPopup(this);"> '
                            % (related_url, url, name))
            extra.append(u'<img src="%s" width="16" height="16" alt="%s" /></a>'
                            % (static('admin/img/selector-search.gif'), _('Lookup')))
        if self.sorted:
            attrs['class'] = '%s sorted' % attrs['class']
        strvalue = ','.join([force_unicode(v) for v in (value or [])])
        output = [super(ForeignKeyRawIdWidget, self).render(name, strvalue, attrs)] + extra
        if value:
            output.append(self.label_for_value(value))
        output.extend(_get_previews(value or [], name))
        return mark_safe(u''.join(output))


from django.forms.widgets import RadioFieldRenderer, RadioInput
from django.utils.encoding import force_unicode

class MediaTypeInput(RadioInput):
    def __unicode__(self):
        if 'id' in self.attrs:
            label_for = ' for="%s_%s"' % (self.attrs['id'], self.index)
        else:
            label_for = ''
        model = get_media_models(*self.choice_value.split('.'))[0]
        output = [
            render_to_string('widgets/mediatypeselect/display_media_type.html', {
                'input': self.tag(),
                'icon_url': os.path.join(
                    settings.MEDIA_URL,
                    media_url('icons/128x128/%s.png' % model.media_type)),
                'label': mark_safe('<label%s>%s</label>' % (label_for, model._meta.verbose_name)),
                'label_for': label_for,
                'help_text': model.help_text,
            }),
        ]
        return mark_safe(''.join(output))


class MediaTypeRenderer(RadioFieldRenderer):
    def __iter__(self):
        for i, choice in enumerate(self.choices):
            yield MediaTypeInput(self.name, self.value, self.attrs.copy(), choice, i)

    def render(self):
        return mark_safe(u'\n'.join([force_unicode(w) for w in self]))


class MediaTypeSelect(forms.RadioSelect):
    renderer = MediaTypeRenderer

    def __init__(self, *args, **kwargs):
        media_models = kwargs.setdefault('media_models', get_media_models())
        del kwargs['media_models']
        kwargs['choices'] = [
            ('%s.%s' % (model._meta.app_label.lower(), model._meta.object_name.lower()),) * 2
            for model in media_models]
        super(MediaTypeSelect, self).__init__(*args, **kwargs)



from django.core.management.base import BaseCommand
from mediastore.mediatypes.video.management import unlock, process_queue


class Command(BaseCommand):
    help = "Process all items stored in the video queue."

    def handle(self, *args, **options):
        if 'unlock' in args:
            unlock()
        process_queue()



from django import forms
from django.contrib import admin
from mediastore.admin import MediaAdmin
from mediastore.mediatypes.audio.models import Audio


class AudioAdmin(MediaAdmin):
    list_display = ('id', 'name', 'file_extension',  'created')
    list_filter = ('file_extension',)


admin.site.register(Audio, AudioAdmin)



# -*- coding: utf-8 -*-
import os
from mediastore.conf import settings

class SettingsProxy(object):
    def __init__(self, settings, defaults):
        self.settings = settings
        self.defaults = defaults

    def __getattr__(self, attr):
        try:
            return getattr(self.settings, attr)
        except AttributeError:
            try:
                return getattr(self.defaults, attr)
            except AttributeError:
                raise AttributeError, 'settings object has no attribute "%s"' % attr


class defaults(object):
    MEDIASTORE_VIDEO_UPLOAD_DIR = os.path.join(
        settings.MEDIASTORE_FS_PREFIX, 'video')
    MEDIASTORE_VIDEO_THUMBNAIL_UPLOAD_DIR = os.path.join(
        settings.MEDIASTORE_FS_PREFIX, 'video', 'thumbnails')
    MEDIASTORE_VIDEO_AUTO_THUMBNAIL_UPLOAD_DIR = os.path.join(
        settings.MEDIASTORE_FS_PREFIX, 'video', 'generated_thumbnails')

    MEDIASTORE_VIDEO_WIDTH = '640'
    MEDIASTORE_VIDEO_HEIGHT = '480'
    MEDIASTORE_VIDEO_BITRATE = '586000'
    MEDIASTORE_VIDEO_AUDIO_BITRATE = '44100'
    MEDIASTORE_VIDEO_THUMBNAIL_POS = '10'
    MEDIASTORE_VIDEO_BACKEND = 'mediastore.mediatypes.video.convert.ffmpeg.FFmpegConverter'

    MEDIASTORE_VIDEO_FFMPEG_CROP = False

    MEDIASTORE_VIDEO_FFMPEG_BIN = 'ffmpeg'
    MEDIASTORE_VIDEO_FFMPEG_ARGS = [
        '-i', '%(input)s',
        '-f', 'flv',
        '-vcodec', 'flv',
        '-acodec', 'adpcm_swf',
        '-b', '%(bitrate)s',
        '-ar', '%(audio_bitrate)s',
        '-croptop', '%(croptop)s',
        '-cropbottom', '%(cropbottom)s',
        '-cropleft', '%(cropleft)s',
        '-cropright', '%(cropright)s',
        '-padtop', '%(padtop)s',
        '-padbottom', '%(padbottom)s',
        '-padleft', '%(padleft)s',
        '-padright', '%(padright)s',
        '-s', '%(width)sx%(height)s',
        '-y', '%(output)s',
    ]
    MEDIASTORE_VIDEO_FFMPEG_THUMBNAIL_BIN = 'ffmpeg'
    MEDIASTORE_VIDEO_FFMPEG_THUMBNAIL_ARGS = [
        '-i', '%(input)s',
        '-f', 'mjpeg',
        '-ss', '%(thumbnail_pos)s',
        '-vframes', '1',
        '-croptop', '%(croptop)s',
        '-cropbottom', '%(cropbottom)s',
        '-cropleft', '%(cropleft)s',
        '-cropright', '%(cropright)s',
        '-padtop', '%(padtop)s',
        '-padbottom', '%(padbottom)s',
        '-padleft', '%(padleft)s',
        '-padright', '%(padright)s',
        '-s', '%(width)sx%(height)s',
        '-an',
        '-y', '%(output)s',
    ]


from django.conf import settings as django_settings
settings = SettingsProxy(django_settings, defaults)
